@@ -10,63 +10,110 @@ DBIx::Simple::Procedure - An Alternative To SQL Stored Procedures using DBIx::Si
 
 =head1 VERSION
 
-Version 1.33
+Version 1.40
 
 =cut
 
-our $VERSION = '1.33';
+our $VERSION = '1.40';
 our @properties = caller();
 
 =head1 SYNOPSIS
 
-This module allow your program to process text files containing one or many commands that execute SQL statements sequentially. Please keep in mind that DBIx::Simple::Procedure
-is an alternative to database stored procedures and not a replacement or emulation of them. Essentially it is an interface to execute and return data from multiple queries.
+This module allow your program to process text files containing one or many commands
+that execute SQL statements sequentially. Please keep in mind that DBIx::Simple::Procedure
+is an alternative to database stored procedures and not a replacement or emulation of
+them. Essentially it is an interface to execute and return data from multiple queries.
 
 Here is an example of how to setup and process a (sql) text file.
 
-    # DBIx::Simple::Procedure uses DBIx::Simple and provides an accessor through the DBIx::Simple::Procedure->{dbix} hash reference.
-    use DBIx::Simple::Procedure;
-    my $db = DBIx::Simple::Procedure->new($path_to_sqlfiles, 'dbi:SQLite:dbname=file.dat'); # will error out using DBIx::Simple if a connection error occurs.
-    
-    # the queue function takes one parameter (a text file) that contains DBIx::Simple::Procedure sql commands.
-    # the process_queue function processes all queued sql statements using the parameters passed to it, similar to the execute function of DBI.
-    $db->queue($sql_file)->process_queue(@sql_parameters);
-    
-    # the cache function returns an array of resultsets, or the resultset of the index passed, return by the select statements encountered in the sql file.
-    # Note! files included using the "include" command will not have there resultsets cached, even if a "capture" command is encountered, only the select statement(s) found in the initial sql file are cached in the order they are encountered.
-    foreach my $result (@{$db->cache(0)}){
-        # do something with the records of the first resultset
-        $result->{...};
-    }
+# DBIx::Simple::Procedure uses DBIx::Simple and provides an accessor through the
+DBIx::Simple::Procedure->{dbix} hash reference.
+
+use DBIx::Simple::Procedure;
+my $db = DBIx::Simple::Procedure->new($path_to_sqlfiles, 'dbi:SQLite:dbname=file.dat');
+# Will error out using DBIx::Simple if a connection error occurs.
+
+# The queue function takes one parameter (a text file) that contains DBIx::Simple::Procedure
+# sql commands.
+
+# The process_queue function processes all queued sql statements using the parameters passed
+# to it, similar to the execute function of DBI.
+
+$db->queue($sql_file)->process_queue(@sql_parameters);
+
+# The cache function returns an array of resultsets, or the resultset of the index passed,
+# return by the select statements encountered in the sql file.
+# Note! files included using the "include" command will not have there resultsets cached,
+# even if a "capture" command is encountered, only the select statement(s) found in the
+# initial sql file are cached in the order they are encountered.
+
+foreach my $result (@{$db->cache(0)}){
+    # do something with the records of the first resultset
+    $result->{...};
+}
 
 =head1 SQL FILE SYNTAX
 
-The (sql) procedural text file to be processed may contain any text you desire, e.g. comments and other markup. DBIx::Simple::Procedure only reacts to command lines (commands).
-These instructions (or commands) must be placed on its own line and be prefixed with an exclamation point, a space, the command, another space, and the statement to be evaluated.
+The (sql) procedural text file to be processed may contain any text you desire, e.g.
+comments and other markup. DBIx::Simple::Procedure only reacts to command lines (commands).
+These instructions (or commands) must be placed on its own line and be prefixed with an
+exclamation point, a space, the command, another space, and the statement to be evaluated.
+
 E.g. "! execute select * from foo".
+
 Multiple commands can be used in a single sql file.
 Note! multi-line sql statements not supported in this release.
 
 SQL File Commands:
     
-    ! execute: This command simply execute the supplied sql statement.
-    ! capture: This is an execute command who's dataset will be cached (stored) for later use. Note! This command can only be used with a select statement.
-    ! replace: This is an execute command that after successfully executed, replaces the scope parameters with data from the last row in its dataset. Note! This command can only be used with a select statement.
-    ! include: This command processes the supplied sql file in a sub transaction. Note! Included sql file processing is isolated from the current processing. Any capture commands encountered in the included sql files will not cache the dataset.
-    ! proceed: This command should be read "proceed if" because it evaluates the string passed (perl code) for truth, if true, it continues if false it skips to the next proceed command or until the end of the sql file.
-    ! ifvalid: This command is a synonym for proceed.
-    ! storage: This command does absolutely nothing except store the sql statement in the commands list (queue) for processing individually from within the perl code with a method like process_command.
-    ! declare: This command is effectively equivalent to the select .. into sql sytax and uses an sql select statement to add vairables to the scope for processing (e.g. ! declare select `name` from `foo` where `id` = $0)
-               can be used in other instructions as $!name, e.g. ! execute update `foo` set `name` = $!name where `id` = $0.
-    ! forward: This command takes an index and jumps to that command line and continues.
-    ! process: This command takes an index and executes that command line.
-    ! examine: This command is used for debugging, it errors out with the compiled statement passed to it.
-    
-    Example: ! execute select * from `foo` where `id` = $0
+! execute:
+            This command simply execute the supplied sql statement.
 
+! capture:
+            This is an execute command who's dataset will be cached (stored) for later use.
+            Note! This command can only be used with a select statement.
+
+! replace:
+            This is an execute command that after successfully executed, replaces the scope
+            parameters with data from the last row in its dataset. Note! This command can
+            only be used with a select statement.
+
+! include:
+            This command processes the supplied sql file in a sub transaction. Note! Included
+            sql file processing is isolated from the current processing. Any capture commands
+            encountered in the included sql files will not cache the dataset.
+
+! proceed:
+            This command should be read "proceed if" because it evaluates the string passed
+            (perl code) for truth, if true, it continues if false it skips to the next proceed
+            command or until the end of the sql file.
+
+! ifvalid:
+            This command is a synonym for proceed.
+
+! storage:
+            This command does absolutely nothing except store the sql statement in the commands
+            list (queue) for processing individually from within the perl code with a method
+            like process_command.
+
+! declare:
+            This command is effectively equivalent to the select .. into sql sytax and uses an
+            sql select statement to add vairables to the scope for processing
+            (e.g. ! declare select `name` from `foo` where `id` = $0) can be used in other
+            instructions as $!name, e.g. ! execute update `foo` set `name` = $!name where `id` = $0.
+
+! forward:
+            This command takes an index and jumps to that command line and continues.
+
+! process:
+            This command takes an index and executes that command line.
+
+! examine:
+            This command is used for debugging, it errors out with the compiled statement passed to it.
+    
 =head1 EXAMPLE
 
-# @@@@@@@@@@ tables/group/insert.sql
+# inside tables/group/insert.sql
 ! include tables/group/create.sql
 ! proceed 1 == 2
 ! execute insert into `group` values (null, concat_ws(' ', 'Test A', $0))
@@ -74,11 +121,12 @@ SQL File Commands:
 ! execute insert into `group` values (null, concat_ws(' ', 'Test B', $0))
 ! proceed 1
 ! capture select * from `group`
-# @@@@@@@@@@ tables/group/create.sql
+
+# inside tables/group/create.sql
 ! execute create table if not exists `group` (`id` int(11) auto_increment, `info` varchar(255) not null, primary key(`id`) )
 ! execute truncate table `group`
 
-# @@@@@@@@@@ test.pl
+# inside test.pl
 use FindBin;
 use DBIx::Simple::Procedure;
 
@@ -91,8 +139,8 @@ my $db = DBIx::Simple::Procedure->new(
 ) or die DBIx::Simple::Procedure->error;
 
 $db->queue('tables/group/insert.sql')->process_queue('One fish, two fish, red fish, blue fish.');
-# or
-$db->queue('tables/group/insert.sql')->process_command(4,'One fish, two fish, red fish, blue fish.'); # execute the 5th command in the tables/group/insert.sql file.
+# or $db->queue('tables/group/insert.sql')->process_command(4,'One fish, two fish, red fish, blue fish.');
+# the process_command method executes the 5th command in the tables/group/insert.sql file.
 
 foreach my $result (@{$db->cache(0)}){
     # $result->{...};
@@ -103,7 +151,10 @@ foreach my $result (@{$db->cache(0)}){
 =cut
 
 =head2 new
-    The new method instantiates a new L<DBIx::Simple> and DBIx::Simple::Procedure object and accepts all parameters required by L<DBIx::Simple>.
+
+The new method instantiates a new DBIx::Simple and DBIx::Simple::Procedure object and accepts all parameters
+required/accepted by DBIx::Simple.
+
 =cut
 
 sub new {
@@ -117,9 +168,7 @@ sub new {
     return $self;
 }
 
-=head2 _load_commands
-    The _load_commands method is an internal method for build the commands dispatch table.
-=cut
+# The _load_commands method is an internal method for build the commands dispatch table.
 
 sub _load_commands {
     my $self = shift;
@@ -127,6 +176,9 @@ sub _load_commands {
     # identify commands that can only contain select statements
     $self->{select_required} = ['capture', 'replace', 'declare'];
     
+    # determine how blank parameters are handled by default
+    $self->{settings}->{blank} = '0';
+    
     #! capture: stores the resultset for later usage
     $self->{commands}->{capture} = sub {
         my ($statement, @parameters) = @_;
@@ -140,13 +192,14 @@ sub _load_commands {
         $self->{processing}->{resultset} = $self->_execute_query($statement, @parameters);
     };
     
-    #! proceed: evaluates the statement passed (perl code) for truth, if true, it continues if false it skips to the next proceed command or until the end of the sql file.
+    #! proceed: evaluates the statement passed (perl code) for truth, if true, it continues if false it
+    #  skips to the next proceed command or until the end of the sql file.
     $self->{commands}->{proceed} = sub {
         my ($statement, @parameters) = @_;
         if (@parameters) {
             foreach my $parameter (@parameters) {
-                $_ = '' if !$_;
-                $statement =~ s/\?/$_/;
+                $parameter = $self->{settings}->{blank} unless defined $parameter;
+                $statement =~ s/\?/$parameter/;
             }
         }
         $self->{processing}->{skip_switch} = eval $statement ? 0 : 1;
@@ -190,8 +243,10 @@ sub _load_commands {
     
     #! forward: changes the queue position, good for looping
     $self->{commands}->{forward} = sub {
+        no warnings;
         my ($statement, @parameters) = @_;
         $self->{cursor} = $statement;
+        next; # purposefully next out of the loop to avoid incrementation. warning should be turned off.
     };
     
     #! process: executes a command in the queue by index number
@@ -210,11 +265,18 @@ sub _load_commands {
         }
         die $self->_error( $statement );
     };
+    
+    #! setting: configures how the module handles blank parameters
+    $self->{commands}->{setting} = sub {
+        my ($statement, @parameters) = @_;
+        $self->{settings}->{blank} = '0' if (lc($statement) eq 'blank as zero');
+        $self->{settings}->{blank} = '' if (lc($statement) eq 'blank as blank');
+        $self->{settings}->{blank} = 'NULL' if (lc($statement) eq 'blank as null');
+    };
 }
 
-=head2 _execute_query
-    The _execute_query method is an internal method for executing queries against the databse in a standardized fashion.
-=cut
+# The _execute_query method is an internal method for executing queries against the databse in
+# a standardized fashion.
 
 sub _execute_query {
     my ($self, $statement, @parameters) = @_;
@@ -222,9 +284,7 @@ sub _execute_query {
     return $resultset;
 }
 
-=head2 _error
-    The die method is an internal method that dies with a standardized error message.
-=cut
+# The _error method is an internal method that dies with a standardized error message.
 
 sub _error {
     my ($self, $message, @parameters) = @_;
@@ -238,61 +298,12 @@ sub _error {
     ( $message || $self->{dbix}->error || "Check the sql file for errors" ) . ".";
 }
 
-=head2 queue
-    The queue function parses the passed (sql) text file and build the list of sql statements to be executed and how.
-=cut
-
-sub queue {
-    my ($self, $sqlfile) = @_;
-    my (@statements);
-    $self->{cmds} = '';
-    
-    # set caller data for error reporting
-    @properties = caller();
-    @statements = $self->_parse_sqlfile($sqlfile);
-    $self->{cmds} = \@statements;
-    return $self;
-}
-
-=head2 process_queue
-    The process_queue function sequentially processes the recorded commands found the (sql) text file.
-=cut
-
-sub process_queue {
-    my ($self, @parameters) = @_;
-    # set caller data for error reporting
-    @properties = caller();
-    $self->{processing}->{parameters} = \@parameters;
-    $self->{processing}->{skip_switch} = 0;
-    $self->{cursor} = 0; 
-    if (@{$self->{cmds}}) {
-        # process sql commands 
-        for (my $i = 0; $self->{cursor} < @{$self->{cmds}}; $i++) {
-            my $cmd = $self->{cmds}->[$self->{cursor}];
-            if ( grep($cmd->{command} eq $_, keys %{$self->{commands}}) )
-            {
-                # process command
-                $self->_processor($cmd);
-                $self->{cursor}++;
-            }
-        }
-        return $self->{processing}->{resultset};
-    }
-    else {
-        die $self->_error( "File has no commands to process" );
-    }
-    return $self;
-}
-
-=head2 _processor
-    The _processor method is an internal methoed that when passed a command hashref, processes the command.
-=cut
+# The _processor method is an internal methoed that when passed a command hashref, processes the command.
 
 sub _processor {
     my ($self, $cmdref) = @_;
     my $command = $cmdref->{command};
     my $statement = $cmdref->{statement};
-    no warnings;
     
     # replace statement placeholders with actual "?" placeholders while building the statement params list
     # my @statement_parameters = map { $self->{processing}->{parameters}[$_] } $statement =~ m/\$(\d+)/g;
@@ -302,7 +313,8 @@ sub _processor {
     # reset statement parameters
     $self->{processing}->{statement_parameters} = ();
     
-    # replace statement placeholders with actual "?" placeholders while building the statement params list using passed or custom parameters
+    # replace statement placeholders with actual "?" placeholders while building the statement params
+    # list using passed or custom parameters
     while ($statement =~ m/(\$\!([a-z0-9A-Z\_\-]+))|(\$(\d+(?!\w)))/) {
         my $custom = $2;
         my $passed = $4;
@@ -318,7 +330,7 @@ sub _processor {
         }
     }
     
-    if ($self->{processing}->{skip_switch} && $command ne "proceed")
+    if ($self->{processing}->{skip_switch} && ( $command ne "proceed" && $command ne "ifvalid" && $command ne "validif" ) )
     {
         # skip command while skip_switch is turned on
         return;    
@@ -331,56 +343,25 @@ sub _processor {
     }
 }
 
-=head2 cache
-    The cache method accesses an arrayref of resultsets that were captured using the (sql file) capture command and return the resultset of the index passed to it or an empty arrayref.
-=cut
+# The _parse_parameters method examines each initially passed in parameter specifically looking for a hashref
+# to add its values to the custom parameters key.
 
-sub cache {
-    my ($self, $index) = @_;
-    return defined $self->{sets}->[$index] ? $self->{sets}->[$index] : [];
-    # return number of cached resultsets if index is not passed.
-    return @{$self->{sets}};
-}
-
-=head2 command
-    The command method is used to queue a command to be processed later by the process_queue method. Takes two arguments, "command" and "sql statement", e.g. command('execute', 'select * from foo').
-=cut
-
-sub command {
-    my ($self, $command, $statement) = @_;
-    my @statements = @{$self->{cmds}};
-    push @statements, { "command" => "$command", "statement" => "$statement" };
-    $self->{cmds} = \@statements;
-    return $self;
-}
-
-=head2 process_command
-    The (*experimental*) process_command method allows you to process the indexed sql satements from your sql file individually. It take one argument, the index of the command as it is encountered in the sql file and tries returns a resultset.
-=cut
-
-sub process_command {
-    my ($self, $index, @parameters) = @_;
-    my $cmd = $self->{cmds}->[$index];
-    if ( grep($cmd->{command} eq $_, keys %{$self->{commands}}) )
-    {
-        # process command
-        $self->{processing}->{parameters} = \@parameters if @parameters;
-        return $self->_processor($cmd);
+sub _parse_parameters {
+    my ($self, @parameters) = @_;
+    for (my $i=0; $i < @parameters; $i++) {
+        my $param = $parameters[$i];
+        if (ref($param) eq "HASH") {
+            while (my($key, $val) = each (%{$param})) {
+                $self->{processing}->{custom_parameters}->{$key} = $val;
+            }
+            delete $parameters[$i];
+        }
     }
-}
-=head2 clear
-    The clear method simply clears the cache (resultset store)
-=cut
-
-sub clear {
-    my $self = shift;
-    $self->{cmds} = '';
+    $self->{processing}->{parameters} = \@parameters;
     return $self;
 }
 
-=head2 _parse_sqlfile
-    The _parse_sqlfile scans the passed (sql) text file and returns a list of sql statement queue objects.
-=cut
+# The _parse_sqlfile method scans the passed (sql) text file and returns a list of sql statement queue objects.
 
 sub _parse_sqlfile {
     my ($self, $sqlfile) = @_;
@@ -404,9 +385,7 @@ sub _parse_sqlfile {
     return @statements;
 }
 
-=head2 _validate_sqlfile
-    The _validate_sqlfile method make sure that the supplied (sql) text file conforms to its command(s) rules.
-=cut
+# The _validate_sqlfile method make sure that the supplied (sql) text file conforms to its command(s) rules.
 
 sub _validate_sqlfile {
     my ($self, @statements) = @_;
@@ -420,6 +399,116 @@ sub _validate_sqlfile {
     }
 }
 
+=head2 queue
+
+The queue function parses the passed (sql) text file and build the list of sql statements to be
+executed and how.
+
+=cut
+
+sub queue {
+    my ($self, $sqlfile) = @_;
+    my (@statements);
+    $self->{cmds} = '';
+    
+    # set caller data for error reporting
+    @properties = caller();
+    @statements = $self->_parse_sqlfile($sqlfile);
+    $self->{cmds} = \@statements;
+    return $self;
+}
+
+=head2 process_queue
+
+The process_queue function sequentially processes the recorded commands found the (sql) text file.
+
+=cut
+
+sub process_queue {
+    my ($self, @parameters) = @_;
+    # set caller data for error reporting
+    @properties = caller();
+    $self->_parse_parameters(@parameters) if @parameters;
+    $self->{processing}->{skip_switch} = 0;
+    $self->{cursor} = 0; 
+    if (@{$self->{cmds}}) {
+        # process sql commands 
+        for (my $i = 0; $self->{cursor} < @{$self->{cmds}}; $i++) {
+            my $cmd = $self->{cmds}->[$self->{cursor}];
+            if ( grep($cmd->{command} eq $_, keys %{$self->{commands}}) )
+            {
+                # process command
+                $self->_processor($cmd);
+                $self->{cursor}++;
+            }
+        }
+        return $self->{processing}->{resultset};
+    }
+    else {
+        die $self->_error( "File has no commands to process" );
+    }
+    return $self;
+}
+
+=head2 cache
+
+The cache method accesses an arrayref of resultsets that were captured using the (sql file) capture command and
+return the resultset of the index passed to it or an empty arrayref.
+
+=cut
+
+sub cache {
+    my ($self, $index) = @_;
+    return defined $self->{sets}->[$index] ? $self->{sets}->[$index] : [];
+    # return number of cached resultsets if index is not passed.
+    return @{$self->{sets}};
+}
+
+=head2 command
+
+The command method is used to queue a command to be processed later by the process_queue method. Takes two
+arguments, "command" and "sql statement", e.g. command('execute', 'select * from foo').
+
+=cut
+
+sub command {
+    my ($self, $command, $statement) = @_;
+    my @statements = @{$self->{cmds}};
+    push @statements, { "command" => "$command", "statement" => "$statement" };
+    $self->{cmds} = \@statements;
+    return $self;
+}
+
+=head2 process_command
+
+The process_command method allows you to process the indexed sql satements from your sql file
+individually. It take two argument, the index of the command as it is encountered in the sql file and tries
+returns a resultset, and any parameters that need to be passed to it.
+
+=cut
+
+sub process_command {
+    my ($self, $index, @parameters) = @_;
+    my $cmd = $self->{cmds}->[$index];
+    if ( grep($cmd->{command} eq $_, keys %{$self->{commands}}) )
+    {
+        # process command
+        $self->_parse_parameters(@parameters) if @parameters;
+        return $self->_processor($cmd);
+    }
+}
+=head2 clear
+
+The clear method simply clears the cache (resultset store)
+
+=cut
+
+sub clear {
+    my $self = shift;
+    $self->{cmds} = '';
+    return $self;
+}
+
 =head1 TODO
 
 ...
@@ -431,8 +520,8 @@ Al Newkirk, C<< <al.newkirk at awnstudio.com> >>
 =head1 BUGS
 
 Please report any bugs or feature requests to C<bug-dbix-simple-procedure at rt.cpan.org>, or through
-the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=DBIx-Simple-Procedure>.  I will be notified, and then you'll
-automatically be notified of progress on your bug as I make changes.
+the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=DBIx-Simple-Procedure>.
+I will be notified, and then you'll automatically be notified of progress on your bug as I make changes.
 
 
 
